﻿Imports System.Collections.ObjectModel
Imports System.Collections.Concurrent
Imports System.Numerics
Imports System.Windows.Controls.DataVisualization.Charting
Imports System.Threading.Tasks
Imports System.Threading

' Parciális osztály, a generált kódtól elkülönül.
Partial Class MainWindow

    ' Konstruktor
    Public Sub New()
        ' Generált kód meghívása
        InitializeComponent()

        ' Inicializálás külön eljárásban
        Benchmark()

    End Sub


    Public Delegate Sub addPointDelegate(x As Double, y As Double)
    Public AddKeyPoint As addPointDelegate
    Public AddFacPoint As addPointDelegate

    ' Olyan lista, amely értesíti a megjelenítőt ha megváltozott az értéke
    Dim factor As New ObservableCollection(Of Plot)
    Dim keygen As New ObservableCollection(Of Plot)

    ' Több szálból is manipulálható sor adatszerkezet
    Dim threadQueue As New ConcurrentQueue(Of BigInteger)

    ' Pont hozzáadása a listához
    Public Sub AddKeyPointMethod(ByVal x As Double, ByVal y As Double)
        keygen.Add(New Plot(x, y))
    End Sub

    ' Pont hozzáadása a listához
    Public Sub AddFacPointMethod(ByVal x As Double, ByVal y As Double)
        factor.Add(New Plot(x, y))
    End Sub

    ' Értékelés indítása
    Public Sub Benchmark()
        ' Faktorizáláshoz szükséges idő grafikon
        Dim series0 As New LineSeries()
        With series0
            .Title = "Faktorizálás"
            .DependentValuePath = "Y"
            .IndependentValuePath = "X"
            .ItemsSource = factor
        End With
        ' Kulcs generáláshoz szükséges idő grafikonja
        Dim series1 As New LineSeries()
        With series1
            .Title = "Kulcs generálás"
            .DependentValuePath = "Y"
            .IndependentValuePath = "X"
            .ItemsSource = keygen
        End With

        ' A grafikonhoz hozzáadjuk a megjelenítendő sorozatokat
        Graph.Series.Add(series0)
        Graph.Series.Add(series1)

        ' Delegáltak definiálása
        AddKeyPoint = AddressOf AddKeyPointMethod
        AddFacPoint = AddressOf AddFacPointMethod

        ' Elindítjuk a taszkokat (lambda függvények segítségével)
        Task.Factory.StartNew(Sub() Calculate(Me))
        Task.Factory.StartNew(Sub() CalculateFact(Me))
    End Sub


    ' Kulcsok számítása
    Public Sub Calculate(ByVal b As MainWindow)
        Dim keySize As Integer = 8
        Dim p, q, n As BigInteger
        Dim sw As New Stopwatch()
        Dim keytime As Double
        ' Végtelen ciklus
        While (True)
            keytime = 0
            ' 3 számítás átlaga
            For i As Integer = 0 To 2
                ' Stopper indítása
                sw.Restart()
                ' Kulcsok generálása
                p = Calculations.generateRandomPrime(keySize)
                q = Calculations.generateRandomPrime(keySize)
                n = (p * q)
                keytime += sw.Elapsed.TotalMilliseconds
                ' Érték hozzáadása a számított érték sorhoz
                threadQueue.Enqueue(n)
            Next
            keySize += 8
            ' Főszálból a megjelenítéshez adás meghívása
            b.Dispatcher.BeginInvoke(AddKeyPoint, keySize, keytime / 3)
        End While
    End Sub

    ' Faktorizálás számítása
    Public Sub CalculateFact(ByVal b As MainWindow)
        Dim keySize As Integer = 8
        Dim p, q, n As BigInteger
        Dim sw As New Stopwatch()
        Dim factime As Double
        ' Végtelen ciklusban
        While (True)
            factime = 0
            ' 3 számítás átlaga
            For i As Integer = 0 To 2
                ' Amíg a sorból nem tudunk értéket kivenni, addig várunk
                While (Not b.threadQueue.TryDequeue(n))
                    Thread.Sleep(25)
                End While
                ' Stopper indítása
                sw.Restart()
                ' Számítás
                p = Calculations.NonTrivialFactor(n)
                q = n / p
                factime += sw.Elapsed.TotalMilliseconds
            Next
            keySize += 8
            ' Főszálból a megjelenítés meghívása
            b.Dispatcher.BeginInvoke(AddFacPoint, keySize, factime / 3)
        End While
    End Sub
End Class
